package com.tzld.piaoquan.recommend.model

import org.apache.spark.rdd.RDD
import org.apache.spark.sql.Row

object MetricUtils {
  def logScale(label: Double, logType: Int, logBase: Double): Double = {
    if (0 == logType) {
      label
    } else {
      Math.log(1 + label) / Math.log(logBase)
    }
  }

  def restoreLog(predict: Double, logType: Int, logBase: Double): Double = {
    if (0 == logType) {
      predict
    } else {
      Math.exp(predict * Math.log(logBase)) - 1
    }
  }

  def clipLabel(label: Double, maxVal: Double = 30): Double = {
    if (label < maxVal) {
      label
    } else {
      maxVal + 2 * Math.log(label - maxVal + 1)
    }
  }

  def calMAPE(evalRdd: RDD[Row]): Double = {
    val apeRdd = evalRdd.map(raw => {
      val label = raw.get(0).toString.toDouble
      val pred = raw.get(1).toString.toDouble
      math.abs(label - pred) / label
    })
    apeRdd.sum() / apeRdd.count()
  }

  def calRMSLE(evalRdd: RDD[Row]): Double = {
    val sleRdd = evalRdd.map(raw => {
      val label = raw.get(0).toString.toDouble
      val pred = raw.get(1).toString.toDouble
      math.pow(math.log(pred + 1) - math.log(label + 1), 2)
    })
    math.sqrt(sleRdd.sum() / sleRdd.count())
  }
}
